#include "LPC21xx.h"
#include <__armlib.h>

/* 16 bit sync value for output file. At least one of the two bits
   not used by "real" data and left low should be high and the rest could
   really be just anything. 
*/
#define SYNC_HIGH   0x06
#define SYNC_LOW    0xb9


extern char asc;
/*
  It seemed foolish to repeat the same code over and over again for
  each ADC conversion so I have created a subroutine to handle it.

  Call with the channel number.

  Channel numbers 0 to 7 use ADC0 and 8 to 15 use ADC1

*/
void do_adc( int channel )
{
  long temp;
  char temp_buf[10];
  int i;

  /* Select channel, clock rate, etc. */

  if(channel >= 0 && channel < 8)   // Use ADC 0
    {
      AD0CR = 0x00200d00 | 1 << channel;

      AD0CR |= 0x01000000;//start conversion
            
      do {
	temp = AD0DR;
      }
      while ((temp & 0x80000000) == 0);
    }
  else if( channel > 7 && channel < 16) // Use ADC 1
    {
      AD1CR = 0x00200d00 | 1 << (channel - 8);

      AD1CR |= 0x01000000;//start conversion
            
      do {
	temp = AD1DR;
      }
      while ((temp & 0x80000000) == 0);
    }

  /* The result is in bits 15:6 so shift right and mask */
  temp >>= 6;
  temp &= 0x3ff;

  if (asc)
    {
      itoa(temp, temp_buf, 10);

      for(i = 0; i < 4; i++)
	if(temp_buf[i] >= '0')
	  out_byte( temp_buf[i]);
      out_byte('\t');    /* Put a tab character between values */
    }       
  else
    {
      out_byte(temp >> 8);
      out_byte(temp);
    }
}





/*
   IRQ routines for ADC operation in burst mode.


   The channel and ADC number are placed in the upper 4 bits of the
   16 bit value written to disk. This is used to demux the data.
 */


void do_adc0_irq(void)
{
  long i;

  i = AD0DR;
  out_byte(((i >> 14)&0x3) | ((i >> 20) & 0x70));
  out_byte(i >> 6);
}

void do_adc1_irq(void)
{
  long i;

  i = AD1DR;
  out_byte(((i >> 14)&0x3) | ((i >> 20) & 0x70) | 0x80);
  out_byte(i >> 6);

}
  
void do_adc_fiq(void)
{
 

}  
