/*
  Interrupt routine "ghetto".

  Turning optimization on breaks interrupt service routines so they
  are segregated here so that the rest of the code can benefit from
  being optimized.

*/
#include "LPC21xx.h"
#include "fat.h"
#include "buffer.h"
#include "adc.h"

#define stat0_on()    (IOCLR0 = 0x04)
#define stat0_off()   (IOSET0 = 0x04)
#define stat1_on()    (IOCLR0 = 0x800)
#define stat1_off()   (IOSET0 = 0x800)

extern char ad1_2, ad1_3, ad1_4, ad1_5, ad1_6, ad1_7;
extern char ad0_1, ad0_2, ad0_3, ad0_4;
extern int  frame;
extern char asc, trig;

void  __attribute__ ((interrupt("IRQ"))) blinkISR(void)
{
  T0IR = 1;   //reset TMR0 interrupt
  VICVectAddr = 0;
}

/*  Stubs for various interrupts (may be replaced later)  */
/*  ----------------------------------------------------  */

void __attribute__ ((interrupt ("IRQ"))) UART0ISR(void)
{
  char temp;


  out_byte(U0RBR);
      
  temp = U0IIR;	//have to read this to clear the int

  /* Update VIC priorities */
  VICVectAddr = 0;

}

/*
  Handled the UART interrupt when in triggered mode. Scans for the 
  trigger character and records the specified number of characters after it.
  It also records the trigger character and appends a carriage return - line feed
  pair at the end.
*/
void  __attribute__ ((interrupt ("IRQ"))) UART0ISR_2(void)
{
  char temp;


  static int count = 0;

  temp = U0RBR;

  if(count == 0)
    if(temp == trig)
      count = frame;

  if( count )
    {
      out_byte(temp);
      count--;
      if(count == 0)
	{
	  out_byte('\n');
	  out_byte('\r');
	}
    }
      
  temp = U0IIR;	//have to read this to clear the int

  /* Update VIC priorities */
  VICVectAddr = 0;
}

void  __attribute__ ((interrupt ("IRQ"))) MODE2ISR(void)
{
#ifdef TESTSD
  static int over = 0;
  static int count = 0;
  static int buf_index = 0;
  long * p;
  int i;

  T0IR = 1;   //reset TMR0 interrupt

 
  if(buffer[buf_index].count == BUF_SIZE)
  {
      count++;
      over++;
  }
  else
    {
      p = (long *)buffer[buf_index].data;
      for(i = 0; i < BUF_SIZE/4; i++)
        *p++ = over ? 0xEFBEADDE : count;
      count++;
      over = 0;
      buffer[buf_index++].count = BUF_SIZE;
      if(buf_index >= NUM_BUFFERS)
	buf_index = 0;
    }
#else    
  T0IR = 1;   //reset TMR0 interrupt
    
  /* Perform the ADC conversions as required
     This probably should be redone to make it table driven
     instead of using this ugly series of if() statements.
   */
    
  if (ad1_5)
      do_adc(8+5);

  if (ad1_4 )
      do_adc(8+4);

  if (ad1_3 )
      do_adc(8+3);

  if (ad0_3)
      do_adc(3);

  if (ad0_2)
      do_adc(2);

  if (ad0_1)
      do_adc(1);

  if (ad1_2)
      do_adc(8+2);

  if (ad0_4)
      do_adc(4);

  if (ad1_7)
      do_adc(8+7);

  if (ad1_6)
    do_adc(8+6);

  /*
    Output some end of line delimiters. A carriage return - line feed
    pair in ASCII mode and "$$" in binary.
  */
  if(asc)
    {
      out_byte('\r');
      out_byte('\n');
    }
  else
    {
      out_byte('$');
      out_byte('$');
    }
#endif	
  VICVectAddr = 0;
	
}
    
void  __attribute__ ((interrupt ("IRQ"))) UART1ISR(void)
{
  char temp;
  int j;

  stat0_on();
  for (j = 0; j < 750000; j++);
  stat0_off();
      
  temp = U1RBR;

      
  temp = U1IIR;	//have to read this to clear the int

  /* Update VIC priorities */
  VICVectAddr = 0;

}



void  __attribute__ ((interrupt("FIQ"))) FIQ_Routine (void)
{
   do_adc_fiq();
  VICVectAddr = 0;     
}


void  __attribute__ ((interrupt("SWI"))) SWI_Routine (void)  {
  while (1) ;	
}


void  __attribute__ ((interrupt("UNDEF"))) UNDEF_Routine (void) {
	
  stat0_on();
  //while (1) ;	
}

/*
  These are just stubs as the main adc code is in another file
  so that compiler code optimization may be turned on for it.
*/

void  __attribute__ ((interrupt("IRQ"))) adc_irq0_handler()
{
  do_adc0_irq();
  VICVectAddr = 0;
}

void  __attribute__ ((interrupt("IRQ"))) adc_irq1_handler()
{
  do_adc1_irq();
  VICVectAddr = 0;
}
